/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.util;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.geotools.factory.GeoTools;
import org.geotools.geometry.jts.JTS;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.geotools.referencing.CRS;
import org.geotools.referencing.ReferencingFactoryFinder;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.CoordinateOperation;
import org.openide.util.Exceptions;

/**
 *
 * @author Gerben Feenstra
 */
public class GeometryUtil {

    private static final AppLogger LOG = AppLogFactory.getLogger(GeometryUtil.class);
    // DO NOT USE -> USE getGeometryFactory()!!
    private static GeometryFactory geometryFactory;
    private static final double ACREAGE_DEVIDER = 10000d;

    /*
     * Uses JTSFactoryFinder to find the GeometryFactory
     */
    private static GeometryFactory getGeometryFactory() {
        if (geometryFactory != null) {
            return geometryFactory;
        } else {
            return (geometryFactory = JTSFactoryFinder.getGeometryFactory(null));
        }
    }

    /**
     * Returns a geometry made from the specified WKT String
     * @param wkt The WKT String
     * @return The Geometry extracted from the String
     */
    public static Geometry wktToGeometry(String wkt) {
        try {

            WKTReader wKTReader = new WKTReader(getGeometryFactory());
            return wKTReader.read(wkt);
        } catch (ParseException ex) {
            String error = "Failed converting WKT to Geometry";
            LOG.error(error, ex);
            throw new WktToGeometryConversionException(error, ex);
        }
    }

    /**
     * Create a Point from a longitude and a latitude value using the specified geometryFactory
     * @param geometryFactory The geometryFactory to use
     * @param longitude 
     * @param latitude
     * @return Point created from the specified coordinates
     */
    public static Point getPoint(GeometryFactory geometryFactory, double longitude, double latitude) {
        Coordinate coordinate = new Coordinate(longitude, latitude);
        Point geometry = geometryFactory.createPoint(coordinate);
        return geometry;
    }

    /**
     * Convenience method, retrieves a GeometryFactory from the JTSFactoryFinder using getGeometryFactory and calls getPoint(GeometryFactory geometryFactory, double longitude, double latitude)
     * @param longitude 
     * @param latitude
     * @return Point created from the specified coordinates
     */
    public static Point getPoint(double longitude, double latitude) {
        return getPoint(getGeometryFactory(), longitude, latitude);
    }

    /**
     * Return acreage in Hectares
     * 
     * @param acreage as provided by vivid solutions Shape
     * @return Acreage in HA
     */
    public static Double getAcreageInHA(Double acreage) {
        return acreage > 0 ? acreage / ACREAGE_DEVIDER : acreage;
    }

    /**
     * Get the acreage from geometry in format for a particular country.
     * As of now it returns the geometry formated in EPSG:28992 or Amersfoort / RD New.
     * TODO: make this method generic and work with some sort of properties file which will give back the appropriate country format
     * http://www.geotoolkit.org/modules/referencing/supported-codes.html
     * @param geometry the geometry to format to the new country format
     * @return the acreage of the formatted geometry.
     * @Throws UnsupportedOperationException when the param geometry is not in the Netherlands
     * @deprecated use GeometryTools.getAcreageFromGeometryUTMZone
     */
    @Deprecated
    public static Double getAcreageFromGeometry(Geometry geometry) {
        Geometry transFormed = null;
        // make geometry for the netherlands
        String netherlandsWKT = "MULTIPOLYGON (((3.10000 51.20000, 3.10000 53.55000, 7.210000 53.55000, 7.21000 50.70000, 3.10000 51.20000)))";
        Geometry netherLands = wktToGeometry(netherlandsWKT);
        if (netherLands.contains(geometry)) {
            try {
                CoordinateReferenceSystem sourceReference = CRS.decode("CRS:84");
                CoordinateReferenceSystem targetReference = CRS.decode("EPSG:28992");

                CoordinateOperation coFactory = ReferencingFactoryFinder.getCoordinateOperationFactory(
                        GeoTools.getDefaultHints()).createOperation(sourceReference, targetReference);

                transFormed = JTS.transform(geometry, coFactory.getMathTransform());
                return transFormed.getArea();
            } catch (org.opengis.referencing.operation.TransformException ex) {
                Exceptions.printStackTrace(ex);
            } catch (MismatchedDimensionException ex) {
                Exceptions.printStackTrace(ex);
            } catch (NoSuchAuthorityCodeException ex) {
                Exceptions.printStackTrace(ex);
            } catch (FactoryException ex) {
                Exceptions.printStackTrace(ex);
            }
            
        }
        return 0D;
    }
    
       
    
}
