/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.util.actions;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.nodes.Node;
import org.openide.util.Lookup;

/**
 * Interface describing the capability to respond to something opened and or
 * closed. <p> Used in the generic {@link OpenAction} to enable all
 * implementations to react on a data object being openend. {@link OpenCloseHandler#close(org.openide.util.Lookup.Provider)
 * }
 * should be called by the data objects editor component. </p>
 * <p>
 *  Typical implementation for a MultiViewElement would be
 * <pre>
 *   public void componentOpened() {
 *       OpenCloseHandler.Event.triggerOpen(obj.getNodeDelegate());
 *   }
 *
 *   public void componentClosed() {
 *       OpenCloseHandler.Event.triggerClose(obj.getNodeDelegate());
 *   }
 * </pre>
 * </>
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public interface OpenCloseHandler {

    /**
     * Handle object with provided context being opened.
     *
     * @param context
     */
    void open(Node context);

    /**
     * Handle object with provided context being closed.
     *
     * @param context
     */
    void close(Node context);

    public static class Event {
        private static final Logger LOGGER = Logger.getLogger(OpenCloseHandler.Event.class.getName());

        /**
         * Trigger an open event and signal all {@link OpenCloseHandler#open(org.openide.util.Lookup.Provider) }
         * implementations
         * 
         * @param context 
         */
        public static void triggerOpen(Node context) {
            for (OpenCloseHandler openCloseHandler : Lookup.getDefault().lookupAll(OpenCloseHandler.class)) {
                try {
                    openCloseHandler.open(context);
                } catch (Exception ex) {
                    LOGGER.log(Level.WARNING, "FIXME:Ignored exception thrown from OpenCloseHandler.open {0}", ex.getMessage());
                }
            }
        }
        
        /**
         * Trigger an open event and signal all {@link OpenCloseHandler#close(org.openide.util.Lookup.Provider) }
         * implementations
         * 
         * @param context 
         */        
        public static void triggerClose(Node context) {
            for (OpenCloseHandler openCloseHandler : Lookup.getDefault().lookupAll(OpenCloseHandler.class)) {
                try {
                    openCloseHandler.close(context);
                } catch (Exception ex) {
                    LOGGER.log(Level.WARNING, "FIXME:Ignored exception thrown from OpenCloseHandler.close {0}", ex.getMessage());
                }
            }
        }        
    }
}
